/*******************************************************************************
 * (c) Copyright 2011 Actel Corporation.  All rights reserved.
 * 
 * This file contains type definitions used throughout G4M Ethernet MAC and PHY
 * device drivers. User need not include this file in application source code.
 * Inclusion of mss_ethernet_mac.h inherits these types.
 * 
 * SVN $Revision: 4791 $
 * SVN $Date: 2012-12-06 10:20:29 +0000 (Thu, 06 Dec 2012) $
 */
#ifndef MSS_ETHERNET_MAC_TYPES_H_
#define MSS_ETHERNET_MAC_TYPES_H_

#include <stddef.h>
#include "../../CMSIS/m2sxxx.h"
#include "mss_ethernet_mac_user_config.h"

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************************
 * Maxmimum interrupts supported in MAC.
 * Two dummy for ease of indexing while access.
 */
#define MSS_MAC_MAX_IRQS                    8u

/*******************************************************************************/
/* Public type definitions                                                     */
/*******************************************************************************/

/*******************************************************************************
 * Ethernet MAC packet buffer type.
 * You need to create two arrays for transmission and reception of packets of
 * this type in application. The size of the arrays must be MSS_MAC_TX_RING_SIZE
 * and MSS_MAC_RX_RING_SIZE for transmission and reception respectively. The
 * driver provides APIs to manage these packets. You must use these APIs while
 * accessing these packets. This is required because the locking mechanism
 * of the packets between the application and driver is maintained by these
 * packet management APIs.
 */
typedef struct
{
    uint8_t lock;  /* Tri-state lock, will have values LOCKED, UNLOCKED or
                      CONSUMED in case of RX and FULL in case of TX */
    uint8_t *buff; /* User must create and maintain
                      [MSS_MAC_TX_RING_SIZE x MSS_MAC_MAX_TX_BUF_SIZE]
                      for transmit packet buffers and
                      [MSS_MAC_RX_RING_SIZE x MSS_MAC_MAX_RX_BUF_SIZE]
                      for receive packet buffers. It is responsibility of the
                      user to keep these buffers always word aligned to
                      meet h/w requirements of MAC engine. */
    uint32_t size; /* No.of bytes of packet data to be transmitted excluding
                      CRC or No.of bytes of packet data received with CRC */
} MAC_pkt_t;

/*******************************************************************************
 * DMA Descriptor
 * Driver creates and manages two descriptor rings for transmission and reception.
 */
typedef struct
{
    uint32_t pkt_start_addr;    /* Packet start address */
    uint32_t pkt_size;          /* Packet size & Per packet override flags */
    uint32_t next_desriptor;    /* Link to next descriptor */
    MAC_pkt_t * pkt;            /* Associated paket */
    uint32_t index;             /* Index: helps in handling interrupts */
} MAC_desc_t;

/*******************************************************************************
 * MAC descriptor lock states
 * These lock states should be used in custom interrupt handlers for MAC.
 */
typedef enum
{
    UNLOCK = 0, /* Freed from DMA but not free to reuse / free */
    LOCK   = 1, /* In use with DMA */
    EMPTY  = 2, /* Can be reused / freed by app */
    FULL   = 3, /* For TX, the data is availabe in pkt buff yet to be transmitted */
}MAC_lock_t;

/*******************************************************************************
 * MAC interrupt definitions
 * This type definition provides interrupt indices for MAC hardware.
 */
typedef enum
{
    MSS_MAC_TXPKTSENT_IRQ = 0u,
    MSS_MAC_TXUNDRRUN_IRQ = 1u,
    MSS_MAC_TXBUSERR_IRQ  = 3u,
    MSS_MAC_RXPKTRCVD_IRQ = 4u,
    MSS_MAC_RXOVRFLOW_IRQ = 6u,
    MSS_MAC_RXBUSERR_IRQ  = 7u,
    MSS_MAC_INVALID_IRQ   = 255u
} MAC_irq_t;
/*******************************************************************************
 * MAC interface speed
 * This type definition provides various interface speeds supported by MAC hardware.
 */
typedef enum
{
    MAC10MBPS     = 0x00,
    MAC100MBPS    = 0x01,
    MAC1000MBPS   = 0x02,
    INVALID_SPEED = 0x03
} MAC_speed_t;

/*******************************************************************************
 * MAC interface type.
 * This type definition provides various interface types for PHY connection.
 */
typedef enum
{
    RMII            = 0x00, 
    RGMII           = 0x01,
    TBI             = 0x02,
    MII             = 0x03,
    GMII            = 0x04,
    INVALID_INF
} MAC_interface_t;

/*******************************************************************************
 * Return type.
 * Return value of APIs is of this type.
 * 1 - on successful execution.
 * 0 - on failure.
 */
typedef uint8_t     return_t;

/*******************************************************************************
 * MAC I/O direction.
 * This type definition provides INPUT or OUTPUT direction of communication.
 */
typedef enum
{
    RX,
    TX,
} MAC_io_t;

/*******************************************************************************
 * This type definition provides various PHY types supported
 */
typedef enum
{
    G4M_SGMII = 0,  /* G4M Ethernet MAC in-built SGMII PHY */
    M88E1111  = 1,  /* Marvell's M88E1111 10/100/1000 Mbps PHY */
    DP83848   = 2,  /* National's DP83848 10/100 Mbps PHY */
    M88E1340  = 3,  /* Marvell's M88E1340 10/100/1000 Mbps PHY */
    AUTO_PHY  = 4,  /* Auto detect / discover the PHY, not supported yet */
    LAST_PHY  = 5   /* Unsupported */
} PHY_type_t;

/*******************************************************************************
 * MAC Configuration Structure.
 * You need to create a record of this type to hold the configuration of the MAC.
 * The MSS_MAC_cfg_struct_def_init() API should be used to initialize the
 * configuration record to default values. Later, the configuration elements
 * in the record can be changed to desired values.
 */
typedef struct
{
    MAC_speed_t speed;                  /* Ethernet Speed: 10/ 100/ 1000 Mbps */
    MAC_interface_t interface;          /* PHY interface type: RMII/ RGMII/ MII /GMII / TBI for SGMII */
    uint8_t auto_negotiation;           /* Enable / disable auto-negotiation */
    uint8_t tx_edc_enable;              /* Enable / disable error detection and correction for tx FIFOs */
    uint8_t rx_edc_enable;              /* Enable / disable error detection and correction for rx FIFOs */
    uint8_t preamble_length;            /* 4-bit Length of preamble field: default value is 0x7 */
    uint8_t interface_mode;             /* Byte / Nibble mode: Always Byte mode for 1000Mbps operation: default is Nibble mode */
    uint8_t hugeframe_enable;           /* Enable / disable huge frame support: default is disable 0 */
    uint8_t length_field_check;         /* Enable / disable length field checking */
    uint8_t pad_n_CRC;                  /* Enable / disable padding and appending CRC */
    uint8_t append_CRC;                 /* Enable / disable appending CRC */
    uint8_t fullduplex;                 /* Enable / disable full duplex: default is disable 0 */
    uint8_t loopback;                   /* Enable / disable loopback mode: default is disable 0 */
    uint8_t rx_flow_ctrl;               /* Enable / disable receiver flow control: default is disable 0 */
    uint8_t tx_flow_ctrl;               /* Enable / disable transmitter flow control: default is disable 0 */
    uint8_t min_IFG;                    /* 8-bit minimum inter-frame gap value */
    uint8_t btb_IFG;                    /* 7-bit back to back inter-frame gap value */
    uint8_t max_retx_tries;             /* 5-bit maximum retransmission tries value: default is 0xF */
    uint8_t excessive_defer;            /* Enable / disable transmission of packet that exceeded max collisions: default is disable 0 */
    uint8_t nobackoff;                  /* Enable / disable back off. default is disable 0 */
    uint8_t backpres_nobackoff;         /* Enable / disable back off in back pressure mode: default is disable 0 */
    uint8_t ABEB_enable;                /* Enable / disable arbitrary binary exponential back-off: default is disable 0 */
    uint8_t ABEB_truncvalue;            /* 4-bit alternative binary exponential back-off value: default is 0xA */
    uint8_t phyclk;                     /* 3-bit MGMT clock divider value */
    uint8_t supress_preamble;           /* Enable / disable preamble suppression at PHY: default is disable 0 */
    uint8_t autoscan_phys;              /* Enable / disable auto scanning of PHYs with programmed addresses: default is disable 0 */
    PHY_type_t phy;                     /* 5-bit PHY address : default is 0x00 */
    uint16_t max_frame_length;          /* Maximum frame length: default value is 0x0600(1536d) */
    uint16_t non_btb_IFG;               /* 14-bit non back to back inter-frame gap value */
    uint16_t slottime;                  /* 10-bit collision window value : default is 0x37 */
    uint32_t framedrop_mask;            /* 18-bit mask to drop frames based on receive statistics */
} MAC_cfg_t;

/*******************************************************************************
 * Transmit and Receive statistics.
 * Statistic of below type, which is desired to be read is to be passed to the
 * function MSS_MAC_read_stat(). The width of returned statistic value is 
 * mentioned in the comment against the statistic.
 */
typedef enum
{
    FRAME_CNT_64,                 /* 18-bit */
    FRAME_CNT_127,                /* 18-bit */
    FRAME_CNT_255,                /* 18-bit */
    FRAME_CNT_511,                /* 18-bit */
    FRAME_CNT_1K,                 /* 18-bit */
    FRAME_CNT_Max,                /* 18-bit */
    FRAME_CNT_VLAN,               /* 18-bit */
    RX_BYTE_CNT,                  /* 24-bit */
    RX_PKT_CNT,                   /* 18-bit */
    RX_FCS_ERR_CNT,               /* 12-bit */
    RX_MULTICAST_PKT_CNT,         /* 18-bit */
    RX_BROADCAST_PKT_CNT,         /* 22-bit */
    RX_CTRL_PKT_CNT,              /* 18-bit */
    RX_PAUSE_PKT_CNT,             /* 12-bit */
    RX_UNKNOWN_OPCODE_CNT,        /* 12-bit */
    RX_ALIGN_ERR_CNT,             /* 12-bit */
    RX_FRAMELENGTH_ERR_CNT,       /* 16-bit */
    RX_CODE_ERR_CNT,              /* 12-bit */
    RX_CS_ERR_CNT,                /* 12-bit */
    RX_UNDERSIZE_PKT_CNT,         /* 12-bit */
    RX_OVERSIZE_PKT_CNT,          /* 12-bit */
    RX_FRAGMENT_CNT,              /* 12-bit */
    RX_JABBER_CNT,                /* 12-bit */
    RX_DROP_CNT,                  /* 12-bit */
    TX_BYTE_CNT,                  /* 24-bit */
    TX_PKT_CNT,                   /* 18-bit */
    TX_MULTICAST_PKT_CNT,         /* 18-bit */
    TX_BROADCAST_PKT_CNT,         /* 18-bit */
    TX_PAUSE_PKT_CNT,             /* 12-bit */
    TX_DEFFERAL_PKT_CNT,          /* 12-bit */
    TX_EXCS_DEFFERAL_PKT_CNT,     /* 12-bit */
    TX_SINGLE_COLL_PKT_CNT,       /* 12-bit */
    TX_MULTI_COLL_PKT_CNT,        /* 12-bit */
    TX_LATE_COLL_PKT_CNT,         /* 12-bit */
    TX_EXCSS_COLL_PKT_CNT,        /* 12-bit */
    TX_TOTAL_COLL_PKT_CNT,        /* 13-bit */
    TX_PAUSE_HONORED_CNT,         /* 12-bit */
    TX_DROP_CNT,                  /* 12-bit */
    TX_JABBER_CNT,                /* 12-bit */
    TX_FCS_ERR_CNT,               /* 12-bit */
    TX_CNTRL_PKT_CNT,             /* 12-bit */
    TX_OVERSIZE_PKT_CNT,          /* 12-bit */
    TX_UNDERSIZE_PKT_CNT,         /* 12-bit */
    TX_FRAGMENT_CNT,              /* 12-bit */
    MAC_LAST_STAT
} MAC_stat_t;

/*******************************************************************************
 * MAC interface status information.
 * This is interface level status information. This can be used to determine
 * link status by calling MSS_MAC_read_interface_status() function.
 */
typedef enum
{
    PE10T_JABBER,     /* PE10T JABBER detected */
    SQE,              /* SQE detected */
    LOSS_OF_CARRIER,  /* Carrier loss detected */
    LINKFAIL,         /* Link fail detected */
    SPEED,            /* 1 - 100Mbps ; 0 - 10Mbps */
    FULLDUPLEX,       /* Full duplex link detected */
    LINKOK,           /* Link is fine */
    JABBER,           /* Jabber frame detected */
    CLASH,            /* Collision detected */
    EXCS_DEFER,       /* Excessive defer detected */
    WOLDTCTD,         /* Wake on LAN magic frame detected (only when WOL integrated ) */
    INVALID_INTF
} MAC_intf_status_t;

/*******************************************************************************
 * PHY indicators.
 * This is PHYsical interface level status information. This can be used to
 * determine present state of PHY control (read/scan/write) operation. This can
 * be used to determine PHY indication which is returned by
 * MSS_MAC_read_phy_indicator() function.
 */
typedef enum
{
    BUSY,    /* PHY is busy, present read/scan cycle is not completed */
    SCAN,    /* PHY is in scan state */
    INVALID, /* Read is invalid */
    LAST_INDICATOR
} MAC_phy_ind_t;

/*******************************************************************************
 * DMA TX / RX status bits
 * These definitions can be used in custom DMA interrupt handlers to determine 
 * the type the DMA interrupt occurred.
 */
typedef enum
{
    TX_PKT_SENT  = 0,
    RX_PKT_RCVD  = 0,
    TX_UNDERRUN  = 1,
    RX_OVERFLOW  = 2,
    TX_BUS_ERROR = 3,
    RX_BUS_ERROR = 3,
    INVALID_DMA_STATUS
} MAC_status_t;

/*******************************************************************************
 * G4Main Ethernet MAC interrupt handler type
 * All default and custom MAC interrupt handlers must of this type.
 */
typedef void (*MAC_callback_t)(void);

/*******************************************************************************
 * G4Main Ethernet MAC instance
 * A local record of this type "g_mac" will be created and maintained by the
 * driver.
 */
typedef struct
{
    MAC_TypeDef         * hw_reg;                            /* MAC registers base: Useful for debugging */
    SYSREG_TypeDef      * sys_reg;                           /* System registers map for debugging */
    IRQn_Type             irqn;                              /* IRQ number assigned to MAC */
    MAC_cfg_t           * cfg;                               /* Pointer to current configuration record */
    MAC_desc_t            tx_desc_tab[MSS_MAC_TX_RING_SIZE]; /* Transmit descriptor table */
    MAC_desc_t          * curr_tx_desc;                      /* Pointer to current transmit descriptor in use */
    MAC_desc_t          * next_tx_desc;                      /* Pointer to next free transmit descriptor */
    MAC_desc_t            rx_desc_tab[MSS_MAC_RX_RING_SIZE]; /* Receive descriptor table */
    MAC_desc_t          * curr_rx_desc;                      /* Pointer to current receive descriptor in use */
    MAC_desc_t          * next_rx_desc;                      /* Pointer to next free receive descriptor */
    PHY_type_t            phy;                               /* Current accessed PHY address */
    uint8_t               mac_addr[6];                       /* Station's MAC address */
    MAC_stat_t            stat;                              /* Latest statistics read / cleared */
    MAC_callback_t        callback[MSS_MAC_MAX_IRQS];        /* callback table for MAC interrupts */
} MAC_instance_t;

/*******************************************************************************
 * PHY device
 * This type definition provides necessary handlers to configure and access 
 * the MDIO compliant PHY devices.
 */
typedef struct 
{
    uint32_t phyid;                                 /* PHY device ID: Vendor specific */
    uint32_t idmask;                                /* PHY ID Mask : Vendor sepecific */
    const uint8_t  * name;                          /* PHY device name: Documentation purpose */
    uint8_t addr;                                   /* PHY Address: H/W design specific  */
    uint32_t supported;                             /* PHY supported features: for advertising */
    return_t (*init)( MAC_cfg_t * cfg );            /* Called to initialize the PHY. MAC_cfg_t * cfg is in-param */
    PHY_type_t (*probe)( void );                    /* Called during discovery of PHY */
    return_t (*config_aneg)( MAC_cfg_t * cfg );     /* Called for auto-negotiation configuration. MAC_cfg_t * cfg is out-param */
    return_t (*read_status)( MAC_cfg_t * cfg );     /* Called to read status. MAC_cfg_t * cfg is out-param */
} PHY_dev_t;

#ifdef __cplusplus
}
#endif

#endif /* MSS_ETHERNET_MAC_TYPES_H_ */


