/*******************************************************************************
 * (c) Copyright 2012 Microsemi Corporation.  All rights reserved.
 *
 *  Microsemi Smartfusion2 USB Device Core driver (LDL layer)  Implementation.	
 *  
 *
 *  Header file for mss_usb_device.c
 *
 * SVN $Revision: 5027 $
 * SVN $Date: 2013-02-01 19:06:39 +0530 (Fri, 01 Feb 2013) $
 */
#ifndef __MSS_USB_DEVICE_H_
#define __MSS_USB_DEVICE_H_

#include "mss_usb_common_cif.h"
#include "mss_usb_device_cif.h"
/***************************************************************************//**
  Constat values which are internally used by the driver.
 */

/* USB request types */
#define USB_STANDARD_REQUEST				            0x00u
#define USB_CLASS_REQUEST					            0x20u
#define USB_VENDOR_REQUEST					            0x40u

/* USB request type masks */
#define USB_STD_REQ_DATA_DIR_MASK				        0x80u
#define USB_STD_REQ_TYPE_MASK					        0x60u
#define USB_STD_REQ_RECIPIENT_MASK				        0x1Fu

#define USB_STD_REQ_DATA_DIR_OUT				        0x00u
#define USB_STD_REQ_DATA_DIR_IN					        0x80u

#define USB_STD_REQ_RECIPIENT_DEVICE                    0x00u
#define USB_STD_REQ_RECIPIENT_INTERFACE                 0x01u
#define USB_STD_REQ_RECIPIENT_ENDPOINT                  0x02u

#define USB_STD_REQ_GET_STATUS                          0x00u
#define USB_STD_REQ_CLEAR_FEATURE                       0x01u
#define USB_STD_REQ_SET_FEATURE                         0x03u
#define USB_STD_REQ_SET_ADDRESS                         0x05u
#define USB_STD_REQ_GET_DESCRIPTOR                      0x06u
#define USB_STD_REQ_SET_DESCRIPTOR                      0x07u
#define USB_STD_REQ_GET_CONFIG							0x08u
#define USB_STD_REQ_SET_CONFIG 		                    0x09u
#define USB_STD_REQ_GET_INTERFACE                       0x0Au
#define USB_STD_REQ_SET_INTERFACE                       0x0Bu
#define USB_STD_REQ_SYNCH_FRAME                         0x0Cu

/* USB Feature selector */
#define USB_STD_FEATURE_REMOTE_WAKEUP					0x0001u
#define USB_STD_FEATURE_TEST_MODE						0x0002u
#define USB_STD_FEATURE_EP_HALT							0x0000u

/* USB Test Mode, Test selectors */
#define USB_TEST_MODE_SELECTOR_TEST_J					0x01u
#define USB_TEST_MODE_SELECTOR_TEST_K					0x02u
#define USB_TEST_MODE_SELECTOR_TEST_SE0NAK				0x03u
#define USB_TEST_MODE_SELECTOR_TEST_PACKET				0x04u
#define USB_TEST_MODE_SELECTOR_TEST_FORCE_ENA			0x05u

/* Descriptor types */
#define USB_DEVICE_DESCRIPTOR_TYPE               		1u
#define USB_CONFIGURATION_DESCRIPTOR_TYPE        		2u
#define USB_STRING_DESCRIPTOR_TYPE               		3u
#define USB_INTERFACE_DESCRIPTOR_TYPE            		4u
#define USB_ENDPOINT_DESCRIPTOR_TYPE             		5u
#define USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE     		6u
#define USB_OTHER_SPEED_CONFIG_DESCRIPTOR_TYPE   		7u
#define USB_INTERFACE_POWER_DESCRIPTOR_TYPE      		8u
#define USB_INTERFACE_ASSOCIATION_DESCRIPTOR_TYPE		11u


#define USB_HID_REPORT_DESCRIPTOR_TYPE              	0x22u

/***************************************************************************//**
  Types which are internally used by the driver.
 ******************************************************************************/
typedef enum {
	MSS_USB_CTRL_XFR_SETUP_PHASE,
	MSS_USB_CTRL_XFR_DATA_IN_PHASE,
	MSS_USB_CTRL_XFR_DATA_OUT_PHASE,
	MSS_USB_CTRL_XFR_HS_PHASE,
	MSS_USB_CTRL_XFR_IDLE_PHASE
} mss_usb_ctrl_xfr_phase_t;

/***************************************************************************//**
  Data structures which Can be used by the class specific components.
 ******************************************************************************/
 
/***************************************************************************//**
 mss_usb_user_descriptors_cb
	The mss_usb_user_descriptors_cb_t provides the prototype of the structure 
	that should be implemented by the application to provide the call-back 
	functions which will be called by the driver to receive the user descriptor 
	information.

	usb_device_descriptor
	This function will be called when USB device receives the GET_DESCRIPTOR 
	command requesting device descriptor from USB host. This function should 
	return a pointer to the device descriptor and provide the length of the 
	descriptor in the return parameter.

	usb_device_qual_descriptor
	This function will be called when USB device receives the GET_DESCRIPTOR 
	command	requesting device qualifier descriptor from USB host. This function 
	should return a pointer to the device qualifier descriptor and provide the 
	length of the descriptor in the return parameter.

	usb_string_descriptor
	This function will be called when USB device receives the GET_DESCRIPTOR 
	command requesting specific string descriptor from USB host. Requested string
	descriptor number is provided in parameter index. This function should return 
	a pointer to the requested string descriptor and provide the length of the 
	descriptor in the return parameter length.
*/
typedef struct mss_usb_user_descriptors_cb {
	uint8_t* (*usb_device_descriptor)(uint32_t* length);
	
	uint8_t* (*usb_device_qual_descriptor)(mss_usb_device_speed_t speed,
										   uint32_t* length);
										   
	uint8_t* (*usb_string_descriptor)(uint8_t index, uint32_t* length);
} mss_usb_user_descriptors_cb_t;

 /***************************************************************************//**
 mss_usb_class_cb
	The mss_usb_user_class_cb_t provides the prototype of the structure that 
	should be implemented by the application/class specific component to provide 
	the call-back functions which will be called by the driver on specific events 
	on the USB bus. 

	usb_class_init
	This function will be called when USB device receives SET_CONFIGURATION 
	request from USB host with a non-zero cfgidx number. The parameter cfgidx 
	indicates the configuration number that should be set by the USB device.
	
	usb_class_deinit
	This function will be called when USB device receives SET_CONFIGURATION 
	request	from USB host with a cfgidx = 0. The parameter cfgidx indicates the 
	configuration number that should be set by the USB device. USB device goes 
	into the MSS_USB_ADDRESS_STATE on receiving this command.
	
	usb_class_get_descriptor
	This function will be called when USB device receives GET_DESCRIPTOR request 
	from USB host requesting a class specific descriptor (configuration, class,
	interface, endpoint, OTG, vendor descriptors).  The parameter recipient 
	indicates the intended recipient by the USB host (endpoint, interface or 
	device). The parameter type indicates the type of descriptor requested.
	
	usb_class_process_request
	This function will be called when USB device receives class specific request 
	from USB host. The parameter request indicates the class specific request 
	that need to be processed. The function should return a pointer to the data 
	requested structure in the return parameter buf_p and length of the buffer 
	in the return parameter length. If the host provides data along with the 
	request to be processed, a pointer to this data will be passed in the 
	parameter data_buf_p, and the size of the buffer will be passed in the 
	parameter data_len.
	
	usb_class_datain
	This function will be called when provided data is transferred on previously
	configured transmit endpoint. The endpoint on which data is received is 
	indicated by parameter num. The parameter status indicates error status of 
	the transmit transaction. A non-zero status value indicates that there was 
	error in last receive transaction.
	
	usb_class_dataout
	This function will be called when data is received on previously configured 
	receive endpoint. The endpoint on which data is received is indicated by 
	parameter num. The parameter status indicates error status of the receive 
	transaction. A non-zero status value indicates that there was error in last 
	receive transaction. The rx_count parameter indicates the number of bytes 
	received in the last receive transaction.
	
	usb_class_cep_datain
	This function will be called when data packet is transmitted on previously 
	configured control endpoint. The parameter status indicates error status of 
	the transmit transaction. A non-zero status value indicates that there was 
	error in last transmit transaction.
	
	usb_class_cep_dataout
	This function will be called when data packet is received on previously 
	configured control endpoint. The parameter status indicates error status 
	of the receive transaction. A non-zero status value indicates that there 
	was error in last receive transaction.

*/
typedef struct mss_usb_class_cb {
	uint8_t (*usb_class_init)(uint8_t cfgidx);
	uint8_t (*usb_class_deinit)(uint8_t cfgidx);
	
	uint8_t* (*usb_class_get_descriptor)(uint8_t recepient,
										 uint8_t type,
										 uint32_t* length);
										 
	uint8_t (*usb_class_process_requests)(uint8_t** buf_p,
										  uint8_t request,
										  uint32_t* length,
										  uint8_t* data_buf_p,
										  uint32_t data_len);
										  
	uint8_t (*usb_class_datain)(mss_usb_ep_num_t num, uint8_t status);
	uint8_t (*usb_class_dataout)(mss_usb_ep_num_t num, uint8_t status, uint32_t rx_count);
	uint8_t (*usb_class_cep_datain)(uint8_t status);
	uint8_t (*usb_class_cep_dataout)(uint8_t status);
} mss_usb_class_cb_t;

/***************************************************************************//**
  Data structures which are internally used by the driver.
 ******************************************************************************/
typedef struct {
    uint8_t request_type;
    uint8_t request;
    uint16_t value;
    uint16_t index;
    uint16_t length;
} mss_usb_device_setup_pkt_t;

#ifdef MSS_USB_DEVICE_TEST_MODE
typedef struct mss_usb_user_device_test_cb {
    void (*usb_user_device_test_ep_dataout)(mss_usb_ep_num_t num, uint8_t status, uint32_t rx_count);
    void (*usb_user_device_test_ep_datain) (mss_usb_ep_num_t num, uint8_t status);

    void (*usb_user_device_test_cep_setup)(uint8_t status);
    void (*usb_user_device_test_cep_dataout)(uint8_t status);
    void (*usb_user_device_test_cep_datain)(uint8_t status);

    void (*usb_user_device_test_sof)(uint8_t status);
    void (*usb_user_device_test_reset)(void);
    void (*usb_user_device_test_suspend)(void);
    void (*usb_user_device_test_resume)(void);
    void (*usb_user_device_test_disconnect)(void);
} mss_usb_user_device_test_cb_t;
#endif

/*******************************************************************************
  Exported functions from Logical device core component which can be used by 
  class specific components.
 *******************************************************************************/

/***************************************************************************//**
  @brief MSS_USB_device_init()
    This API must be used by the application to initialize the USB driver in device 
    mode. Appropriate value of the parameter must be provided to make the driver 
    operate at desired USB speed.
  
  @param speed
    This parameter specifies the USB speed at which the USB driver and the MSS USB
    core should operate. 
    Valid values for this parameter are
		MSS_USB_DEVICE_HS
		MSS_USB_DEVICE_FS

  @return
    This function does not return a value.

  Example:
  @code
  @endcode
*/
void 
MSS_USB_device_init
(
    mss_usb_device_speed_t speed
);

/***************************************************************************//**
  @brief MSS_USB_device_set_desc_cb_handler()
    This API must be used by the application to provide the call-back functions
	to the driver which will be called by this driver to get the USB user 
	descriptors (Descriptors which are not lass specific) from the application.
	This API must be called before initializing the USB Device Core Driver.
  
  @param user_desc_cb
    This parameter provides the address of the structure of type 
	mss_usb_user_descriptors_cb_t which is implemented by the class driver or 
	application.

  @return
    This function does not return a value.

  Example:
  @code
  @endcode
*/
void 
MSS_USB_device_set_desc_cb_handler
(
    mss_usb_user_descriptors_cb_t* user_desc_cb
);

/***************************************************************************//**
  @brief MSS_USB_device_set_class_cb_handler()
    This API must be used by the application to provide the call-back functions
	to the driver which will be called by this driver to indicate the specific 
	events happening on the USB Bus. This API must be called before initializing 
	the USB Device Core Driver.
  
  @param speed
    This parameter provides the address of the structure of type mss_usb_class_cb_t
	which is implemented by the class driver.

  @return
    This function does not return a value.

  Example:
  @code
  @endcode
*/
void 
MSS_USB_device_set_class_cb_handler
(
    mss_usb_class_cb_t* class_cb
);

/***************************************************************************//**
  @brief MSS_USB_device_cep_configure()
    This API configures the Control Endpoint (EP0) for the Control transactions 
	and enables the Control endpoint interrupt. All Properties of the Control 
	endpoint are fixed as listed below.
		Transfer type			 Control Transfer.
		Maximum Packet length 	 0x40 (64 decimal)
		FIFO address 			 0x00
		FIFO Size 				- 0x40 (64 decimal)
	
	This API must be called before any other API for the control endpoint.

  @param 
    This function does not take any parameters.
  
  @return
    This function does not return a value.

  Example:
  @code
  @endcode
*/
void
MSS_USB_device_cep_configure
(
    void
);

/***************************************************************************//**
  @brief MSS_USB_device_cep_read_prepare()
    This function prepares the previously configured Control endpoint for 
	receiving data. After calling this function, USB device is ready to read data
	received on the next OUT packet. This function will store the data read from 
	endpoint FIFO into the provided buffer. If the data in receive FIFO exceeds 
	in length than the length parameter value then this function read data equal 
	to length value.
	
  @param addr
    This parameter specifies the address of the receive buffer where the data 
	from RX FIFO is stored by the driver.
  
  @param length
    This parameter specifies the length of the receive buffer in bytes.
  
  @return
    This function does not return a value.

  Example:
  @code
  @endcode
*/
void
MSS_USB_device_cep_read_prepare
(
	uint8_t * addr,
	uint32_t length
);

/***************************************************************************//**
  @brief MSS_USB_device_cep_write()
    This API loads the Control Endpoint FIFO with the data from provided transmit
	buffer. On receiving the next IN token, the data will be transmitted on the 
	USB bus. If the length parameter value is more than the size of the TX FIFO 
	then this functions loads number of bytes equal to the length parameter value.
	
  @param addr
    This parameter specifies the address of the buffer from which the data is to 
	be copied to the Control endpoint buffer.
    
  @param length
    This parameter specifies the length of the provided buffer in bytes.
	
  @return
    This function does not return a value. 

  Example:
  @code
  @endcode
*/
void
MSS_USB_device_cep_write
(
	uint8_t * addr,
	uint32_t length
);

/***************************************************************************//**
  @brief MSS_USB_device_cep_error()
    This API is used to send a stall condition on control endpoint when the USB 
	request is not supported or device is not able to respond to the request.
	
  @param 
	This function does not take any parameters.
  
  @return
    This function does not return a value.

  Example:
  @code
  @endcode
*/
void
MSS_USB_device_cep_error
(
	void
);

/***************************************************************************//**
  @brief MSS_USB_device_cep_clr_error()
    This API is used to clear a stall condition on previously stalled control 
	endpoint.
	
  @param 
    This function does not take any parameters
    
  @return
    This function does not return a value.

  Example:
  @code
  @endcode
*/
void 
MSS_USB_device_cep_clr_error
(
	void
);

/***************************************************************************//**
  @brief MSS_USB_device_tx_ep_configure()
    This API configures the transmit endpoint with the configuration values 
	provided as parameters. After initializing the core, this API must be called
	before any other API for the desired transmit endpoint.
	
	Note:  To provide complete flexibility in configuring the FIFOs, this driver 
	does not make any assumptions on the FIFO size and FIFO address. User is 
	responsible to make sure that the endpoint FIFO does not overlap with other 
	endpoint FIFO space.
  
  @param ep_num
    This parameter is the endpoint number on which USB transfers are to be 
	performed. The endpoint number must be of  mss_usb_ep_num_t type.
  
  @param fifo_addr
    This parameter is the address of the FIFO in the internal RAM in the USB 
	core. Valid FIFO address values are from 0x00 to 0xFFF8. FIFO Address must be 
	multiple of 8. If the value provided is not multiple of 8, then the immediate 
	lower value which is multiple of 8 is taken as the FIFO address. E.g. If the 
	provided value is 0x09, the actual value taken by driver is 0x08. If the 
	provided value is less than 0x08 then FIFO address is taken as 0x00.

  @param fifo_size
    This parameter provides the endpoint FIFO size in the USB core internal RAM. 
	Valid FIFO size values are 8, 16, 32, 64, 128, 512, 1024, 2048, 4096 . MSS USB
	core by default assigns 8 byte FIFO if the FIFO size is not configured.
  
  @param max_pkt_size
    This parameter provides the maximum packet size of the USB transfer. This 
	value must be equal to the maximum packet size as mentioned in the endpoint 
	descriptor which is used during enumeration process.
	
	Note: Make sure that this value is less than or equal to the FIFO size value.

  @param num_usb_pkt
    This parameter has different meanings for different types of transfers.
	
	Low bandwidth ISO/interrupt transfers  This parameter must always be 1u. 
	This parameter represents the number of packets transferred in one (micro) 
	frame.
	
	High bandwidth ISO transfers  This parameter represents the number of packets 
	transferred in one (Micro) frame. In this case, this parameter can have a 
	value of 1 2 or 3. High bandwidth ISO transfers are yet implemented.
	
	Bulk transfers.  For Bulk transfer this value must always be 1u. This 
	parameter will be used with the auto-amalgamation/auto-split feature where 
	it indicates number of bulk packets to be auto-amalgamated/auto-split in bulk 
	transfer. 
	The auto-amalgamation/auto-split feature is implemented but not yet tested.

  @param dma_enable
	This parameter specifies if internal DMA should be used for the data transfer
	from the provided buffer to the USB FIFO or not.
	
  @param dma_channel
	This parameter specifies the internal DMA channel to be used for the data 
	transfers. DMA channel is associated with selected endpoint. Unique DMA channel
	should be selected to transfer data on individual endpoints. This parameter is 
	ignored when dma_enable parameter indicates that the DMA should not be used.
  
  @param xfr_type
   This parameter specifies the type of transfer to be performed on the selected 
   endpoint. Except control transfer, all other types of transfers (Interrupt, 
   Isochronous, and Bulk) can be selected.
   
   @param add_zlp
	This parameter indicates whether a Zero length packet (ZLP) should be sent if
	the transfer is exact multiple of wMaxPacketSize. This parameter is only 
	applicable for Bulk transfers. For all other transfer types this parameter 
	is ignored.


  @return
	This function does not return a value.

  Example:
  @code
  @endcode
*/
void
MSS_USB_device_tx_ep_configure
(
	mss_usb_ep_num_t ep_num,
	uint16_t fifo_addr,
	uint16_t fifo_size,
	uint16_t max_pkt_size,
	uint8_t num_usb_pkt,
	uint8_t dma_enable,
	mss_usb_dma_channel_t dma_channel,
    mss_usb_xfr_type_t xfr_type,
    uint32_t add_zlp
);

/***************************************************************************//**
  @brief MSS_USB_device_rx_ep_configure()
    This API configures the receive endpoint with the configuration values 
	provided as parameters. After initializing the core, this API must be called
	before any other API for the desired receive endpoint.
	
	Note:  To provide complete flexibility in configuring the FIFOs, this driver
	does not make any assumptions on the FIFO size and FIFO address. User is 
	responsible to make sure that the endpoint FIFO does not overlap with other 
	endpoint FIFO space.

  
  @param ep_num
    This parameter is the endpoint number on which USB transfers are to be 
	performed. The endpoint number must be of  mss_usb_ep_num_t type.
  
  @param fifo_addr
    This parameter is the address of the FIFO in the internal RAM in the USB 
	core. Valid FIFO address values are from 0x00 to 0xFFF8. FIFO Address must be 
	multiple of 8. If the value provided is not multiple of 8, then the immediate 
	lower value which is multiple of 8 is taken as the FIFO address. E.g. If the 
	provided value is 0x09, the actual value taken by driver is 0x08. If the 
	provided value is less than 0x08 then FIFO address is taken as 0x00.

  @param fifo_size
    This parameter provides the endpoint FIFO size in the USB core internal RAM. 
	Valid FIFO size values are 8, 16, 32, 64, 128, 512, 1024, 2048, 4096 . MSS USB
	core by default assigns 8 byte FIFO if the FIFO size is not configured.
  
  @param max_pkt_size
    This parameter provides the maximum packet size of the USB transfer. This 
	value must be equal to the maximum packet size as mentioned in the endpoint 
	descriptor which is used during enumeration process.
	
	Note: Make sure that this value is less than or equal to the FIFO size value.

  @param num_usb_pkt
    This parameter has different meanings for different types of transfers.
	
	Low bandwidth ISO/interrupt transfers  This parameter must always be 1u. 
	This parameter represents the number of packets transferred in one (micro) 
	frame.
	
	High bandwidth ISO transfers  This parameter represents the number of packets 
	transferred in one (Micro) frame. In this case, this parameter can have a 
	value of 1 2 or 3. High bandwidth ISO transfers are yet implemented.
	
	Bulk transfers.  For Bulk transfer this value must always be 1u. This 
	parameter will be used with the auto-amalgamation/auto-split feature where 
	it indicates number of bulk packets to be auto-amalgamated/auto-split in bulk 
	transfer. 
	The auto-amalgamation/auto-split feature is implemented but not yet tested.

  @param dma_enable
	This parameter specifies if internal DMA should be used for the data transfer
	from the provided buffer to the USB FIFO or not.
	
  @param dma_channel
	This parameter specifies the internal DMA channel to be used for the data 
	transfers. DMA channel is associated with selected endpoint. Unique DMA channel
	should be selected to transfer data on individual endpoints. This parameter is 
	ignored when dma_enable parameter indicates that the DMA should not be used.
  
  @param xfr_type
   This parameter specifies the type of transfer to be performed on the selected 
   endpoint. Except control transfer, all other types of transfers (Interrupt, 
   Isochronous, and Bulk) can be selected.

   @param add_zlp
	This parameter indicates whether to expect a Zero length packet (ZLP) if the
	transfer is exact multiple of wMaxPacketSize. This parameter is only 
	applicable for Bulk transfers. For all other transfer types this parameter 
	is ignored.

  @return
	This function does not return a value.
  

  Example:
  @code
  @endcode
*/

void
MSS_USB_device_rx_ep_configure
(
	mss_usb_ep_num_t ep_num,
	uint16_t fifo_addr,
	uint16_t fifo_size,
	uint16_t max_pkt_size,
	uint8_t num_usb_pkt,
	uint8_t dma_enable,
	mss_usb_dma_channel_t dma_channel,
    mss_usb_xfr_type_t xfr_type,
    uint32_t add_zlp
);

/***************************************************************************//**
  @brief MSS_USB_device_rx_ep_read_prepare()
    This API prepares the previously configured receive endpoint for USB receive
	operation. After calling this function, data will be received on the selected
	endpoint when the next OUT packet is received.

	Note: Make sure that MSS_USB_device_rx_ep_configure() is called before this
	API is called.

  @param ep_num
	This parameter specifies the endpoint number on which the data is to be received.
	
  @param addr
	This parameter is the address of the buffer to which the received data is 
	copied from the selected endpoint buffer.
	
  @param length
	This parameter specifies the length of the buffer in bytes.
  
  @return
	This function does not return a value.

  Example:
  @code
  @endcode
*/
void
MSS_USB_device_rx_ep_read_prepare
(
	mss_usb_ep_num_t ep_num,
	uint8_t* addr,
	uint32_t length
);

/***************************************************************************//**
  @brief MSS_USB_device_tx_ep_write()
    This API writes the data provided by user into the previously configured 
	endpoint FIFO. After calling this API, the core transmits the data in EP FIFO
	on receiving the next IN token. After complete data has been transferred 
	callback function will be called to indicate successful data-in transaction.
	
  @param ep_num
	This parameter specifies the endpoint number on which the data is to be 
	transmitted.
	
  @param addr
	This parameter is the address of the buffer provided by user from which the 
	data is copied to the selected endpoint buffer.
  
  @param Length
	This parameter specifies the length of the buffer provided.
  
  @return
	This function does not return a value.
  

  Example:
  @code
  @endcode
*/
void
MSS_USB_device_tx_ep_write
(
	mss_usb_ep_num_t ep_num,
	uint8_t * addr,
	uint32_t length
);

/***************************************************************************//**
  @brief MSS_USB_device_tx_ep_error()
	This API is used to send a stall condition on selected transmit endpoint.
  
  @param ep_num
	This parameter specifies the transmit endpoint number which is to be stalled.
  
  @return
	This function does not return a value.
  

  Example:
  @code
  @endcode
*/
void 
MSS_USB_device_tx_ep_error
(
	mss_usb_ep_num_t ep_num
);

/***************************************************************************//**
  @brief MSS_USB_device_rx_ep_error()
	This API is used to send a stall condition on selected receive endpoint.
	
  @param ep_num
	This parameter specifies the receive endpoint number which is to be stalled.
  
  @return
	This function does not return a value.
  
  Example:
  @code
  @endcode
*/
void 
MSS_USB_device_rx_ep_error
(
	mss_usb_ep_num_t ep_num
);

/***************************************************************************//**
  @brief MSS_USB_device_tx_ep_clr_error()
	This API is used to clear a stall condition on previously stalled transmit 
	endpoint.
	
  @param ep_num
	This parameter specifies the transmit endpoint number whose stall condition 
	is to be cleared.
	
  @return
	This function does not return a value.

  Example:
  @code
  @endcode
*/
void 
MSS_USB_device_tx_ep_clr_error
(
	mss_usb_ep_num_t ep_num
);

/***************************************************************************//**
  @brief MSS_USB_device_rx_ep_clr_error()
	This API is used to clear a stall condition on previously stalled receive 
	endpoint.
  
  @param ep_num
	This parameter specifies the receive endpoint number whose stall condition 
	is to be cleared.
  
  @return
	This function does not return a value.
  
  Example:
  @code
  @endcode
*/
void 
MSS_USB_device_rx_ep_clr_error
(
	mss_usb_ep_num_t ep_num
);

#endif  /* __MSS_USB_DEVICE_H_ */
