/*******************************************************************************
 * (c) Copyright 2012 Microsemi Corporation.  All rights reserved.
 *
 *  Microsemi Smartfusion2 USB device logical driver layer Implementation.	
 *  
 *
 *  Header file for mss_usb_device_cdc.c
 *
 * SVN $Revision: 5027 $
 * SVN $Date: 2013-02-01 19:06:39 +0530 (Fri, 01 Feb 2013) $
 */

#ifndef __MSS_USB_DEVICE_CDC_H_
#define __MSS_USB_DEVICE_CDC_H_

#include <stdint.h>
#include "mss_usb_device.h"
/*******************************************************************************
 * Public definitions from CDC class driver
 */
/*******************************************************************************
   CDC Class configuration definitions: These values will be used by CDC driver
   to configure the MSS USB core endpoints. These values are alos used for
   generating the descriptor tables which are sent to the USB host durring 
   enumeration.
  
   Note:    Application developers are allwed to modify these vallues according 
   to requirement. 
   Make sure that the values here are in accordance with the user descriptors
   (device descriptor etc), USB 2.0 standard and USB Core Driver operation speed
   (MSS_USB_DEVICE_HS or MSS_USB_DEVICE_FS)
 */

/*MSS USB core Endpoints used by CDC class*/
#define VCP_BULK_DATA_OUT_EP 			            MSS_USB_RX_EP_1
#define VCP_BULK_DATA_IN_EP  			            MSS_USB_TX_EP_2
#define VCP_INTR_COMM_IN_EP  			            MSS_USB_TX_EP_1

/*MSS USB core FIFO address for endpoints used by CDC class*/      
#define VCP_DATA_TX_EP_FIFO_ADDR                    0x100u
#define VCP_DATA_RX_EP_FIFO_ADDR                    0x400u
#define VCP_COMM_TX_EP_FIFO_ADDR                    0x50u 

/*MSS USB core FIFO size for endpoints used by CDC class*/   
#define VCP_DATA_TX_EP_FIFO_SIZE                    512u
#define VCP_DATA_RX_EP_FIFO_SIZE                    512u
#define VCP_COMM_TX_EP_FIFO_SIZE                    64u

/*Maximum packet size for endpoints used by CDC class*/   
#define VCP_DATA_TX_EP_MAX_PKT_SIZE                 512u
#define VCP_DATA_RX_EP_MAX_PKT_SIZE                 512u
#define VCP_COMM_TX_EP_MAX_PKT_SIZE                 64u

/*******************************************************************************
 * Private definitions used by CDC class driver
 */

/* Class specific Descriptor lengths */
#define CONFIGURATION_DESCR_LENGTH      			9u
#define INTERFACE_DESCR_LENGTH          			9u
#define INTERFACE_ASSOCIATION_DESCR_LENGTH          8u
#define ENDPOINT_DESCR_LENGTH           			7u

/*Full configuration descriptor length. */
#define FULL_CONFIG_DESCR_LENGTH                    75u 

/* CDC Requests for ACM sub-class */
#define SEND_ENCAPSULATED_COMMAND                   0x00
#define GET_ENCAPSULATED_RESPONSE                   0x01
#define SET_COMM_FEATURE                            0x02
#define GET_COMM_FEATURE                            0x03
#define CLEAR_COMM_FEATURE                          0x04
#define SET_LINE_CODING                             0x20
#define GET_LINE_CODING                             0x21
#define SET_CONTROL_LINE_STATE                      0x22
#define SEND_BREAK                                  0x23
#define NO_CMD                                      0xFF

/*******************************************************************************
 * Public constants andd ata strucures from CDC class driver.
 */
/***************************************************************************//**
  The usb_cdc_state_t is used to provide information of current state of the
  CDC class driver.
  USB_CDC_NOT_CONFIGURED -- USB CDC class driver is not configured and it can 
                            not perform	data transfers.
  USB_CDC_CONFIGURED    --  USB CDC class driver is configured by Host and it 
                            can perform data data transfers.
  USB_CDC_TRANSMITTING  --  USB CDC class driver is configured by Host and is 
                            transmitting data.
  USB_CDC_RECEIVING  --     USB CDC class driver is configured by Host and is 
                            receiving data.
*/
typedef enum usb_cdc_state {
	USB_CDC_NOT_CONFIGURED,
    USB_CDC_CONFIGURED,
	USB_CDC_TRANSMITTING,
    USB_CDC_RECEIVING
}usb_cdc_state_t;

/***************************************************************************//**
  The mss_usb_cdc_app_cb_t provides the prototype for all the call-back 
  function handler which should be implemented by the user application. User 
  application must define and initialize a structure of this type and assign 
  the address of that structure as parameter in MSS_USB_CDC_init() function.

  usb_cdc_init
  The function pointed by this function pointer will be called to indicate that 
  the CDC driver is configured by the USB host. The application can used this 
  function to prepare itself for data exchange with the host. This driver can not 
  perform data transfers unless it is configured by the USB host to do so.

  usb_cdc_release
  The function pointed to by this function pointer will be called when the host 
  wants to un-configure this CDC device.

  usb_cdc_process_request
  The function pointed by this function pointer will be called when this driver 
  receives a CDC class specific request to be processed. The request is provided 
  to the application in the parameter request. If additional data was sent by 
  host along with the request, a pointer to this data is provided in data_buf_p 
  parameter and length of his data is provided in data_len parameter. When the 
  application wants to send data to host in response to a request it must provide 
  a pointer to the data to be sent in parameter buf and the length of this buffer 
  in bytes must be provided in parameter length.

  usb_cdc_tx_complete
  The function pointed by this function pointer will be called to indicate 
  completion of data transmission operation. The status parameter provides the
  error status of this operation. A non-zero value indicates that there was an 
  error in last operation.

  usb_cdc_rx
  The function pointed by this function pointer will be called to indicate data 
  is available to read from RX EP FIFO. The status parameter provides the error 
  status of this operation. A non-zero value indicates that there was an error 
  in last operation. Number of bytes received during this operation is indicated 
  in rx_count parameter.

  usb_cdc_notification
  This driver needs to transmit CDC class Notification data on the Interrupt 
  endpoint on a periodic basis. The period of transmission of data should be 
  selected in endpoint descriptor. Host gets this information during the 
  enumeration process. Application should provide the notification data to this 
  driver by implementing usb_cdc_notification element of 
  mss_usb_cdc_app_cb_t type structure. This callback function is called 
  periodically when IN token is received on CDC interrupt endpoint.
*/
typedef struct usb_device_cdc_vcp_app_cb {
	void (*usb_cdc_init)(void);
	void (*usb_cdc_release)(void);
	
    uint8_t (*usb_cdc_process_request)(uint8_t** buf,
                                       uint8_t request,
                                       uint32_t* length,
                                       uint8_t* data_buf_p,
                                       uint32_t data_len);
    
	uint8_t (*usb_cdc_tx_complete)(uint8_t status);
	uint8_t (*usb_cdc_rx)(uint8_t status, uint32_t rx_count);
	void (*usb_cdc_notification)(uint8_t** buf_p, uint32_t* length_p);
} mss_usb_cdc_app_cb_t;

extern mss_usb_cdc_app_cb_t* mss_usb_cdc_vcp_ops;

/*******************************************************************************
 * Exported APIs CDC VCP driver
 ******************************************************************************/
 
/***************************************************************************//**
  @brief MSS_USB_CDC_init()
	This API must be used by the application to start initialize the CDC class
	driver. A pointer to the structure of type mss_usb_cdc_app_cb_t must be passed
	 as a parameter to this function. 
  
  @param vcp_app_cb
	A pointer to the structure of type mss_usb_cdc_app_cb_t. This will be used by 
	the CDC class driver to call the call-back functions implemented by application.

  @return
	This function does not return a value.
  
  Example:
  @code
  @endcode
*/
void
MSS_USB_CDC_init
(
    mss_usb_cdc_app_cb_t* vcp_app_cb
);

/***************************************************************************//**
  @brief MSS_USB_CDC_tx()
	This API can be used by the application to start transmission on the CDC VCP.
	A pointer to the buffer which needs to be transmitted should be provided in 
	the parameter buf. Length of the buffer should be provided in the parameter 
	length.
  
  @param buf
    A pointer to the buffer which needs to be transmitted should be provided in 
	this parameter.
	
  @param length
	Number of data byte to be transmitted should be provided in this parameter.
	
  @return
	This function does not return a value.
  
  Example:
  @code
  @endcode
*/
void
MSS_USB_CDC_tx
(
	uint8_t* buf,
	uint32_t length
);

/***************************************************************************//**
  @brief MSS_USB_CDC_rx_prepare()
    This API can be used by the application to prepare CDC VCP to receive data. 
	Once prepared, this driver will write data into the provided buffer.
	
  @param buf
    A pointer to the buffer in which the data should be stored should be provided
	in this parameter.
  
  @param length
    Length of the buffer, provided in buf parameter, should be provided in this 
	parameter.
	
  @return
	This function does not return a value.
  
  Example:
  @code
  @endcode
*/
void
MSS_USB_CDC_rx_prepare
(
	uint8_t* buf,
	uint32_t length
);

/***************************************************************************//**
  @brief MSS_USB_CDC_get_state()
	This API can be used by the application to know the current state of the 
	CDC Class driver.
	
  @param 
	This function does not take any parameter.
	  
  @return
	This function returns the status of the CDC Class driver of type 
	usb_cdc_state_t.
  
  Example:
  @code
  @endcode
*/
usb_cdc_state_t
MSS_USB_CDC_get_state
(
	void
);

#endif  /* __MSS_USB_DEVICE_CDC_H_ */
