/*******************************************************************************
 * (c) Copyright 2012 Microsemi Corporation.  All rights reserved.
 *
 *  Microsemi Smartfusion2 USB device core interface driver layer Implementation.	
 *  
 *
 *  This file implements MSS USB core initialization in device mode, and acts as
 *  core interface for the logical layer to communicate with the MSS USB core.
 *
 * SVN $Revision: 5027 $
 * SVN $Date: 2013-02-01 19:06:39 +0530 (Fri, 01 Feb 2013) $
 */

#include "../../CMSIS/mss_assert.h"
#include "mss_usb_core_regs.h"
#include "mss_usb_common_cif.h"
#include "mss_usb_device_cif.h"
#include "mss_usb_device_reg_io.h"
#include "mss_usb_common_reg_io.h"

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************************
 These pointers are used for holding the Receive transfer parametes (Buffer 
 address, length) etc. This pointer will be updated in 
 MSS_USB_core_device_rx_ep_read_prepare() API and used in 
 MSS_USB_core_handle_rx_ep_irq().
*/
volatile mss_usb_ctrl_ep_state_t ctrl_ep_state;

/*******************************************************************************
* This structure is used for tracking the Control endoint states..
*/
mss_usb_ep_t *core_receive_ep[5];
mss_usb_ep_t *core_transmit_ep[5];

/*******************************************************************************
 * Private functions declarations in USB Device core interface component.
 *******************************************************************************/

/*******************************************************************************
 * This function initializes the MUSBMHDRC core in Device mode.
 */
void
MSS_USB_core_device_init
(
    mss_usb_device_speed_t speed
)
{
    ctrl_ep_state = MSS_USB_CTRL_EP_IDLE;

    /*TODO: Avoid resetting twice in DualRole mode*/
    MSS_USB_core_soft_reset();
	MSS_USB_core_clr_usb_irq();
  	NVIC_ClearPendingIRQ(USB_IRQn);
    
    /*Reset and Resume are by defual enabled in INTRUSBE register after soft reset.*/
	NVIC_EnableIRQ(USB_IRQn);	

  	NVIC_ClearPendingIRQ(USB_DMA_IRQn);
	NVIC_EnableIRQ(USB_DMA_IRQn);	
	
	MSS_USB_core_rx_ep_disable_irq_all();
	MSS_USB_core_tx_ep_disable_irq_all();
    
	if(speed == MSS_USB_DEVICE_FS)
	{
		MSS_USB_core_disable_hs_mode();
	}
}
	
/*******************************************************************************
 Sets the address of the device as received in the Set_Address command
 */
void
MSS_USB_core_device_set_address
(
    uint8_t addr
)
{
    USB->FADDR = addr;
}

/*******************************************************************************
 Soft connect -- D+ and D- lines are connected to the bus
 Valid in Device mode only    
 */
void
MSS_USB_core_device_connect
(
    void
)
{
    USB->POWER |= POWER_REG_SOFT_CONN_MASK;
}

/*******************************************************************************
 Soft diconnect -- D+ and D- lines are dis connected from the bus
 Valid in Device mode only
*/
void
MSS_USB_core_device_disconnect
(
    void
)
{
    USB->POWER &= ~POWER_REG_SOFT_CONN_MASK;
}

/*******************************************************************************
 * EXPORTED APIs
 *******************************************************************************/

/*******************************************************************************
 Provides the information about the core configuration
 */
mss_usb_core_info_t
MSS_USB_core_get_device_info(void)
{
    mss_usb_core_info_t temp;

    temp.core_max_nbr_of_tx_ep =(USB->EP_INFO & 0x0Fu);
    temp.core_max_nbr_of_rx_ep =((USB->EP_INFO & 0xF0u) >> 4u);
    temp.core_ram_bus_width =(USB->RAM_INFO & 0x0Fu);
	temp.core_max_nbr_of_dma_chan =((USB->RAM_INFO & 0xF0u) >> 4u);
    temp.core_WTCON =((USB->LINK_INFO & 0xF0u) >> 4u);
    temp.core_WTID =(USB->LINK_INFO & 0x0Fu);
    temp.core_VPLEN = USB->VP_LEN;
    temp.core_HS_EOF1 = USB->HS_EOF1;
    temp.core_FS_EOF1 = USB->FS_EOF1;
    temp.core_LS_EOF1 = USB->LS_EOF1;

    return(temp);
}

/*******************************************************************************
 Configures the registers related to TX EP for data transfer operations as 
 per the parameters provided by the upper layer. 
 */
void
MSS_USB_core_device_tx_ep_configure
(
	mss_usb_ep_t* device_ep
)
{
	MSS_USB_core_tx_ep_clr_under_run(device_ep->num);
	MSS_USB_core_tx_ep_clr_stall(device_ep->num);
	MSS_USB_core_tx_ep_clr_stall_sent(device_ep->num);

	switch(device_ep->xfr_type)
	{
		case MSS_USB_XFR_INTERRUPT:
            MSS_USB_core_tx_ep_disable_iso(device_ep->num);
            MSS_USB_core_tx_ep_clr_autoset(device_ep->num);
		break;	

		case MSS_USB_XFR_BULK:			
            MSS_USB_core_tx_ep_disable_iso(device_ep->num);
            MSS_USB_core_tx_ep_set_autoset(device_ep->num);
		break;	

		case MSS_USB_XFR_ISO:	
            /*Error check and Data toggle is ignored in ISO transfers*/
            MSS_USB_core_tx_ep_enable_iso(device_ep->num);
            MSS_USB_core_tx_ep_clr_autoset(device_ep->num);
		break;	

        default:
            ASSERT(0);
		break;
	}
    
    /*Do the common configuration for TX EP*/
	MSS_USB_core_common_tx_ep_configure(device_ep);
}

/*******************************************************************************
 Configures the RX EP for data transfer operations as per the parameters 
 provided by upper layer.
 */
void
MSS_USB_core_device_rx_ep_configure
(
	mss_usb_ep_t* device_ep
)
{
	/*Device mode configurations*/
	MSS_USB_core_rx_ep_clr_overrun(device_ep->num);
	MSS_USB_core_rx_ep_clr_stall_sent(device_ep->num);
	MSS_USB_core_rx_ep_clr_send_stall(device_ep->num);
	
	switch(device_ep->xfr_type)
	{
		case MSS_USB_XFR_INTERRUPT:		        
			MSS_USB_core_rx_ep_disable_iso(device_ep->num);
			MSS_USB_core_rx_ep_disable_nyet(device_ep->num);
            MSS_USB_core_rx_ep_clr_autoclr(device_ep->num);
        break;

		case MSS_USB_XFR_BULK:            
			MSS_USB_core_rx_ep_disable_iso(device_ep->num);
			MSS_USB_core_rx_ep_enable_nyet(device_ep->num);
			MSS_USB_core_rx_ep_set_autoclr(device_ep->num);
		break;	
		
        case MSS_USB_XFR_ISO:
            MSS_USB_core_rx_ep_clr_autoclr(device_ep->num);
			MSS_USB_core_rx_ep_enable_iso(device_ep->num);
			MSS_USB_core_rx_ep_disable_nyet(device_ep->num);
		break;	

        default:
            ASSERT(0);
        break;
	}

  	/*Do the common configuration for RX EP*/
	MSS_USB_core_common_rx_ep_configure(device_ep);
}

/*******************************************************************************
 Configures the Control EP for data transfer operations.
 */
void
MSS_USB_core_device_cep_configure
(
	void
)
{
	/*Control transfers will be handled without DMA*/
	MSS_USB_core_set_index_reg(MSS_USB_CTRL_EP);
	MSS_USB_core_cep_reset_csr0_reg();
	MSS_USB_core_cep_enable_irq();
}

/*******************************************************************************
 Prepares the Control EP for receving data as per the parameters provided by 
 upper layer
 */
void
MSS_USB_core_device_cep_rx_prepare
(
	mss_usb_ep_t* device_ep
)
{
	if(MSS_USB_CEP_IDLE == device_ep[MSS_USB_CTRL_EP].state)
	{
		ctrl_ep_state = MSS_USB_CTRL_EP_IDLE;
	}
	else if((MSS_USB_CEP_SETUP == device_ep[MSS_USB_CTRL_EP].state) ||
           (MSS_USB_CEP_DATAOUT == device_ep[MSS_USB_CTRL_EP].state))
	{
		ctrl_ep_state = MSS_USB_CTRL_EP_RX;
	}	
}

/*******************************************************************************
 Prepares the RX EP for receving data as per the parameters provided by upper layer
 */
void
MSS_USB_core_device_rx_ep_read_prepare
(
	mss_usb_ep_t* device_ep
)
{
    /*
     Store the endpoint info to calculate number of bytes transfered by
     DMA and to update the txn_length and xfr_length.
    */
	core_receive_ep[(device_ep->num)] = device_ep;
	MSS_USB_core_rx_ep_enable_irq(device_ep->num);
    if((device_ep->dma_enable == DMA_ENABLE))
    {  
        /*Make sure that address is Modulo-4.Bits D0-D1 are read only.*/
		ASSERT(!(((uint32_t)device_ep->buf_addr) & 0x00000002));
        MSS_USB_core_dma_write_addr(device_ep->dma_channel, (uint32_t)device_ep->buf_addr);

        /*
        DMA Count register will be loaded after receive interrupt occures.
        Mode need to be set everytime since M1 to M0 transition might have
        happened for "short packet".
        */
         if((device_ep->xfr_type == MSS_USB_XFR_BULK))
        {	
			MSS_USB_core_rx_ep_set_dma_mode1(device_ep->num);
            MSS_USB_core_rx_ep_set_autoclr(device_ep->num);
            MSS_USB_core_rx_ep_enable_dma(device_ep->num);

            MSS_USB_core_dma_write_count(device_ep->dma_channel , device_ep->xfr_length);

            /*Handling single NULL packet reception*/
            if(0u != device_ep->xfr_length )
            {
                MSS_USB_core_dma_start_xfr(device_ep->dma_channel);
            }
        }
        else
        {
            MSS_USB_core_rx_ep_clr_autoclr(device_ep->num);
            MSS_USB_core_rx_ep_set_dma_mode0(device_ep->num);
            MSS_USB_core_rx_ep_disable_dma(device_ep->num);
        }
    }
}

/*******************************************************************************
 Forces Device to resume when in suspend mode.
 */
void 
MSS_USB_core_device_force_resume
(
	void
)
{
	MSS_USB_core_device_resume_gen();
}


/*******************************************************************************
 ISO EP Will wait for SOF packet after TxPktRdy is set 
 */
void
MSS_USB_core_device_set_isoupdate
(
    void
)
{
    USB->POWER |= POWER_REG_ISO_UPDATE_MASK;
}

/*******************************************************************************
 * ISO EP Will NOT wait for SOF packet after TxPktRdy is set and will transmit 
 * data if IN token is received in same frame.
 */
void
MSS_USB_core_device_clr_isoupdate
(
    void
)
{
    USB->POWER &= ~POWER_REG_ISO_UPDATE_MASK;
}

/*******************************************************************************
 Clears a stall on RX EP.
 */
void
MSS_USB_core_rx_ep_clr_stall
(
	mss_usb_ep_num_t ep_num
)
{
	MSS_USB_core_rx_ep_clr_send_stall(ep_num);
    
	/*TODO: Cinfirm...this is required for clear feature command*/	
	MSS_USB_core_rx_ep_clr_data_tog(ep_num);
}

/*******************************************************************************
 Sends a stall on Control EP.
 */
void
MSS_USB_core_cep_stall
(
	void
)
{
	/*TODO: check if we need to clear rxpktrdy here*/
	MSS_USB_core_cep_send_stall();
}

/*******************************************************************************
 Sends a stall on TX EP.
 */
void
MSS_USB_core_tx_ep_stall
(
	mss_usb_ep_num_t ep_num
)
{
	/*TODO: to wait for last packet or not should be decided by App.*/
    while(MSS_USB_core_ep_is_txfifo_notempty(ep_num));

    MSS_USB_core_tx_ep_set_send_stall(ep_num);
}

/*******************************************************************************
 Sends a stall on RX EP.
 */
void
MSS_USB_core_rx_ep_stall
(
	mss_usb_ep_num_t ep_num
)
{
	MSS_USB_core_rx_ep_clr_rxpktrdy(ep_num);
	MSS_USB_core_rx_ep_set_send_stall(ep_num);		
}

/*******************************************************************************
 Clears a stall on Control EP.
 */
void
MSS_USB_core_cep_clr_stall
(
	void
)
{
	/*TODO: This API is not needed.SendStall bit is Autoclearing for Control EP*/
/*	MSS_USB_core_cep_clr_stall_sent();*/
}

/*******************************************************************************
 * Clears a stall on TX EP.
 */
void
MSS_USB_core_tx_ep_clr_stall
(
	mss_usb_ep_num_t ep_num
)
{
	MSS_USB_core_tx_ep_clr_send_stall(ep_num);
	/*
	TODO: Cinfirm...this is required for cleare feature command
	Lakeview page:62
	*/	
	MSS_USB_core_tx_ep_clr_data_tog(ep_num);
}

/*******************************************************************************
 Writes a data packet on EP0 in device mode(control endpoint).
 */
void
MSS_USB_core_device_cep_write_pkt
(
	mss_usb_ep_t* device_ep
)
{
	if(device_ep->num == MSS_USB_CTRL_EP)
	{
		/* null buffer, xfr type, transaction type */
		if((device_ep->buf_addr != 0) &&(device_ep->xfr_type == MSS_USB_XFR_CONTROL))
		{
			MSS_USB_core_load_tx_fifo(device_ep->num,
                                      device_ep->buf_addr,
                                      device_ep->txn_length);
            
			device_ep->txn_count = device_ep->txn_length;
			device_ep->xfr_count += device_ep->txn_length;
		}
		
        if(MSS_USB_CEP_DATAIN == device_ep->state)
		{
			ctrl_ep_state = MSS_USB_CTRL_EP_TX;
		}
		
        MSS_USB_core_set_index_reg(MSS_USB_CTRL_EP);
		MSS_USB_core_cep_set_txpktrdy();
		
        if(device_ep->xfr_count >= device_ep->xfr_length)
		{
			MSS_USB_core_cep_set_dataend();
		}
	}	
}

/*******************************************************************************
 Reads data packet arrived on EP0 in device mode(control endpoint).
 */
void
MSS_USB_core_device_cep_read_pkt
(
	mss_usb_ep_t* device_ep
)
{
   	volatile uint16_t received_count = 0u;
   	MSS_USB_core_set_index_reg(MSS_USB_CTRL_EP);

	if((device_ep->num == MSS_USB_CTRL_EP) &&(device_ep->buf_addr != 0))
	{
		/*TODO: check stalled, null buffer, xfr type, transaction type */
		received_count = MSS_USB_core_cep_rx_byte_count();
		
        MSS_USB_core_read_rx_fifo(MSS_USB_CTRL_EP, device_ep->buf_addr, received_count);
		
        device_ep->xfr_count += received_count;
		device_ep->txn_count = received_count;
        MSS_USB_core_cep_clr_rxpktrdy();
        
        /*Dataend bit should not be set when we unload Setup Packet.*/
		if(ctrl_ep_state == MSS_USB_CTRL_EP_IDLE)
		{
            /*set Dataend if Length in setup packet is zero (No data phase)*/
			if((device_ep->buf_addr[6] == 0) &&(device_ep->buf_addr[7] == 0))
			{
				MSS_USB_core_cep_set_dataend();
			}
		}
		else if(ctrl_ep_state == MSS_USB_CTRL_EP_RX)
		{
			if(device_ep->xfr_count == device_ep->xfr_length)
			{
				MSS_USB_core_cep_set_dataend();
			}
		}
		else
		{
		    ASSERT(0);
		}
	}	
}

#ifdef __cplusplus
}
#endif
